/* Copyright (C) Eliot Stock - All Rights Reserved
 * Unauthorized copying of this file, via any medium is strictly prohibited.
 * Proprietary and confidential.
 * Written by Eliot Stock, 1@eliotstock.com
 */
#ifndef ACC_KIONIX_KTXJ2_H_
#define ACC_KIONIX_KTXJ2_H_

// Datasheet: http://kionixfs.kionix.com/en/datasheet/KXTJ2-1009%20Specifications%20Rev%209.pdf
// Source of register details: http://www.kionix.com/downloads

/* registers */
//output register x
#define KXTJ2_OUTX_L 0x06
//
#define KXTJ2_OUTX_H 0x07
//output register y
#define KXTJ2_OUTY_L 0x08
//
#define KXTJ2_OUTY_H 0x09
//output register z
#define KXTJ2_OUTZ_L 0x0A
//
#define KXTJ2_OUTZ_H 0x0B
//communication selftest
#define KXTJ2_DCST_RESP 0x0C
//WHO_AM_I
#define KXTJ2_WHO_AM_I 0x0f
//direction source of taptap
#define KXTJ2_INT_SOURCE1 0x16
//positions source of wakeup event
#define KXTJ2_INT_SOURCE2 0x17
//status (main int)
#define KXTJ2_STATUS_REG 0x18
//functions interrupt latch  release
#define KXTJ2_INT_REL 0x1A
//1' control register
#define KXTJ2_CTRL_REG1 0x1B
//2' control register
#define KXTJ2_CTRL_REG2 0x1D
//control for physical interrupt pin
#define KXTJ2_INT_CTRL_REG1 0x1E
//interrupt control for directions and motion
#define KXTJ2_INT_CTRL_REG2 0x1F
//4' control register
#define KXTJ2_DATA_CTRL_REG 0x21
//wakeup control timer
#define KXTJ2_WAKEUP_TIMER 0x29
//charge type self test
#define KXTJ2_SELF_TEST 0x3A
//wakeup threshold
#define KXTJ2_WAKEUP_THRESHOLD 0x6A

/* registers bits */
//before set
#define KXTJ2_DCST_RESP_COM_TEST_BEFORE (0x55 << 0)
//after set
#define KXTJ2_DCST_RESP_COM_TEST_AFTER (0xaa << 0)
//WHO_AM_I -value
#define KXTJ2_WHO_AM_I_COMPONENT_ID (0x9 << 0)
//datareadyint
#define KXTJ2_INT_SOURCE1_DRDY (0x01 << 4)
//wakeup status event
#define KXTJ2_INT_SOURCE1_WUFS (0x01 << 1)
//x-
#define KXTJ2_INT_SOURCE2_XNWU (0x01 << 5)
//x+
#define KXTJ2_INT_SOURCE2_XPWU (0x01 << 4)
//y-
#define KXTJ2_INT_SOURCE2_YNWU (0x01 << 3)
//y+
#define KXTJ2_INT_SOURCE2_YPWU (0x01 << 2)
//z-
#define KXTJ2_INT_SOURCE2_ZNWU (0x01 << 1)
//z+
#define KXTJ2_INT_SOURCE2_ZPWU (0x01 << 0)
//on/off
#define KXTJ2_CTRL_REG1_PC (0x01 << 7)
//low/high performance
#define KXTJ2_CTRL_REG1_RES (0x01 << 6)
//data ready enable
#define KXTJ2_CTRL_REG1_DRDYE (0x01 << 5)
//2g range
#define KXTJ2_CTRL_REG1_GSEL_GR2 (0x0 << 3)
//4g range
#define KXTJ2_CTRL_REG1_GSEL_GR4 (0x1 << 3)
//8g range
#define KXTJ2_CTRL_REG1_GSEL_GR8 (0x2 << 3)
//8g range with 14b resolution
#define KXTJ2_CTRL_REG1_GSEL_GR8_14 (0x3 << 3)
//wakeup enable
#define KXTJ2_CTRL_REG1_WUFE (0x01 << 1)
//soft reset
#define KXTJ2_CTRL_REG2_SRST (0x01 << 7)
//command test control
#define KXTJ2_CTRL_REG2_DCST (0x01 << 4)
//0.78Hz
#define KXTJ2_CTRL_REG2_OWUF_0P781 (0x0 << 0)
//1.563Hz
#define KXTJ2_CTRL_REG2_OWUF_1P563 (0x1 << 0)
//3.125Hz
#define KXTJ2_CTRL_REG2_OWUF_3P125 (0x2 << 0)
//6.25Hz
#define KXTJ2_CTRL_REG2_OWUF_6P25 (0x3 << 0)
//12.5Hz
#define KXTJ2_CTRL_REG2_OWUF_12P5 (0x4 << 0)
//25Hz
#define KXTJ2_CTRL_REG2_OWUF_25 (0x5 << 0)
//50Hz
#define KXTJ2_CTRL_REG2_OWUF_50 (0x6 << 0)
//100Hz
#define KXTJ2_CTRL_REG2_OWUF_100 (0x7 << 0)
//INT enable
#define KXTJ2_INT_CTRL_REG1_IEN (0x01 << 5)
//INT polarity
#define KXTJ2_INT_CTRL_REG1_IEA (0x01 << 4)
//INT latching
#define KXTJ2_INT_CTRL_REG1_IEL (0x01 << 3)
//selftest polarity
#define KXTJ2_INT_CTRL_REG1_STPOL (0x01 << 1)
//x-
#define KXTJ2_INT_CTRL_REG2_XNWU (0x01 << 5)
//x+
#define KXTJ2_INT_CTRL_REG2_XPWU (0x01 << 4)
//y-
#define KXTJ2_INT_CTRL_REG2_YNWU (0x01 << 3)
//y+
#define KXTJ2_INT_CTRL_REG2_YPWU (0x01 << 2)
//z-
#define KXTJ2_INT_CTRL_REG2_ZNWU (0x01 << 1)
//z+
#define KXTJ2_INT_CTRL_REG2_ZPWU (0x01 << 0)
//12.5Hz
#define KXTJ2_DATA_CTRL_REG_OSA_12P5 (0x0 << 0)
//25Hz
#define KXTJ2_DATA_CTRL_REG_OSA_25 (0x1 << 0)
//50Hz
#define KXTJ2_DATA_CTRL_REG_OSA_50 (0x2 << 0)
//100Hz
#define KXTJ2_DATA_CTRL_REG_OSA_100 (0x3 << 0)
//200Hz
#define KXTJ2_DATA_CTRL_REG_OSA_200 (0x4 << 0)
//400Hz
#define KXTJ2_DATA_CTRL_REG_OSA_400 (0x5 << 0)
//800Hz
#define KXTJ2_DATA_CTRL_REG_OSA_800 (0x6 << 0)
//1600Hz
#define KXTJ2_DATA_CTRL_REG_OSA_1600 (0x7 << 0)
//0.78Hz
#define KXTJ2_DATA_CTRL_REG_OSA_0P781 (0x8 << 0)
//1.563Hz
#define KXTJ2_DATA_CTRL_REG_OSA_1P563 (0x9 << 0)
//3.125Hz
#define KXTJ2_DATA_CTRL_REG_OSA_3P125 (0xa << 0)
//6.25Hz
#define KXTJ2_DATA_CTRL_REG_OSA_6P25 (0xb << 0)
//6.25Hz
#define KXTJ2_DATA_CTRL_REG_OSA_LP_6P25 (0x0 << 0)
//12.5Hz
#define KXTJ2_DATA_CTRL_REG_OSA_LP_12P5 (0x1 << 0)
//25Hz
#define KXTJ2_DATA_CTRL_REG_OSA_LP_25 (0x2 << 0)
//50Hz
#define KXTJ2_DATA_CTRL_REG_OSA_LP_50 (0x3 << 0)
//100Hz
#define KXTJ2_DATA_CTRL_REG_OSA_LP_100 (0x4 << 0)
//200Hz
#define KXTJ2_DATA_CTRL_REG_OSA_LP_200 (0x5 << 0)
//400Hz
#define KXTJ2_DATA_CTRL_REG_OSA_LP_400 (0x6 << 0)
//800Hz
#define KXTJ2_DATA_CTRL_REG_OSA_LP_800 (0x7 << 0)
//0.3905Hz
#define KXTJ2_DATA_CTRL_REG_OSA_LP_0P3905 (0x8 << 0)
//0.78Hz
#define KXTJ2_DATA_CTRL_REG_OSA_LP_0P781 (0x9 << 0)
//1.563Hz
#define KXTJ2_DATA_CTRL_REG_OSA_LP_1P563 (0xa << 0)
//3.125Hz
#define KXTJ2_DATA_CTRL_REG_OSA_LP_3P125 (0xb << 0)
//charge on
#define KXTJ2_SELF_TEST_ENABLE (0x01 << 1)
//charge off
#define KXTJ2_SELF_TEST_DISABLE (0x01 << 4)

/*registers bit masks */
//
#define KXTJ2_DCST_RESP_COM_TEST_MASK 0xff
//
#define KXTJ2_WHO_AM_I_COMPONENT_MASK 0xff
//None
#define KXTJ2_INT_SOURCE1_DRDY_MASK 0x10
//None
#define KXTJ2_INT_SOURCE1_WUFS_MASK 0x2
//None
#define KXTJ2_INT_SOURCE2_XNWU_MASK 0x20
//None
#define KXTJ2_INT_SOURCE2_XPWU_MASK 0x10
//None
#define KXTJ2_INT_SOURCE2_YNWU_MASK 0x8
//None
#define KXTJ2_INT_SOURCE2_YPWU_MASK 0x4
//None
#define KXTJ2_INT_SOURCE2_ZNWU_MASK 0x2
//None
#define KXTJ2_INT_SOURCE2_ZPWU_MASK 0x1
//
#define KXTJ2_STATUS_REG_INT_MASK 0x10
//None
#define KXTJ2_CTRL_REG1_PC_MASK 0x80
//None
#define KXTJ2_CTRL_REG1_RES_MASK 0x40
//None
#define KXTJ2_CTRL_REG1_DRDYE_MASK 0x20
//grange selection 1
#define KXTJ2_CTRL_REG1_GSEL_MASK 0x18
//None
#define KXTJ2_CTRL_REG1_WUFE_MASK 0x2
//None
#define KXTJ2_CTRL_REG2_SRST_MASK 0x80
//None
#define KXTJ2_CTRL_REG2_DCST_MASK 0x10
//wake-up ODR
#define KXTJ2_CTRL_REG2_OWUF_MASK 0x7
//None
#define KXTJ2_INT_CTRL_REG1_IEN_MASK 0x20
//None
#define KXTJ2_INT_CTRL_REG1_IEA_MASK 0x10
//None
#define KXTJ2_INT_CTRL_REG1_IEL_MASK 0x8
//None
#define KXTJ2_INT_CTRL_REG1_STPOL_MASK 0x2
//None
#define KXTJ2_INT_CTRL_REG2_XNWU_MASK 0x20
//None
#define KXTJ2_INT_CTRL_REG2_XPWU_MASK 0x10
//None
#define KXTJ2_INT_CTRL_REG2_YNWU_MASK 0x8
//None
#define KXTJ2_INT_CTRL_REG2_YPWU_MASK 0x4
//None
#define KXTJ2_INT_CTRL_REG2_ZNWU_MASK 0x2
//None
#define KXTJ2_INT_CTRL_REG2_ZPWU_MASK 0x1
//main stream odr
#define KXTJ2_DATA_CTRL_REG_OSA_MASK 0xf
//low power stream odr
#define KXTJ2_DATA_CTRL_REG_OSA_LP_MASK 0xf
//None
#define KXTJ2_SELF_TEST_ENABLE_MASK 0x2
//None
#define KXTJ2_SELF_TEST_DISABLE_MASK 0x10

// Everything below is our own, not from Kionix.

// From the product spec:
//   "The KXTJ2’s Slave Address is comprised of a programmable part and a fixed part, which allows for connection of multiple KXTJ2's to
//   the same I2C bus. The Slave Address associated with the KXTJ2 is 000111X, where the programmable bit, X, is determined by the
//   assignment of ADDR (pin 1) to GND or IO_Vdd."
// We have out ADDR pin connected to ground. Right shift to knock the R/W bit off.
#define KIONIX_SLAVE_ADDRESS (0b00011100 >> 1)

// Wait for 500 ms of motion before firing the interrupt. This is number of clock cycles at the OWUF frequency we set.
//   Timer counts = time (s) * frequency (Hz)
//                = 0.4 s * 50 Hz
//                = 20 (decimal)
//                = 11001 (binary)
//     0b0001 1001: 500ms
//     0b0001 0100: 400ms
//     0b0000 1111: 300ms
//     0b0000 1010: 200ms
//     0b0000 0101: 100ms
#define KIONIX_TIME_FOR_ALERTING (0b00010100)
//     0b0000 0010:  40ms
#define KIONIX_TIME_FOR_TRACKING (0b00000010)

// Wait for 0.5g of acceleration for the above duration before firing the interrupt. This is in "counts" where there are 16 counts per g.
//   Threshold counts = acceleration (g) * 16
//                    = 0.5 * 16
//                    = 8 (decimal)
//                    = 1000 (binary)
#define KIONIX_THRESHOLD_FOR_ALERTING (0b00001000)
#define KIONIX_THRESHOLD_FOR_TRACKING (0b00001000)

#define KIONIX_MAX_PENDING_TRANSACTIONS (40)

typedef enum {
    KIONIX_MODE_ALERTING,
    KIONIX_MODE_TRACKING
} kionix_mode_t;

void on_acc_interrupt_pin_high(nrf_drv_gpiote_pin_t pin, nrf_gpiote_polarity_t action);
void acc_kionix_init(void);
void acc_kionix_program(kionix_mode_t mode);
void acc_kionix_clear_interrupt(void);

#endif /* ACC_KIONIX_KTXJ2_H_ */
